#JES- Jython Environment for Students
#Copyright (C) 2002  Jason Ergle, Claire Bailey, David Raines, Joshua Sklare
#See JESCopyright.txt for full licensing information

import javax.swing as swing
import java
import java.lang as lang

ACTIVATED = swing.event.HyperlinkEvent.EventType.ACTIVATED
ENTERED = swing.event.HyperlinkEvent.EventType.ENTERED
EXITED = swing.event.HyperlinkEvent.EventType.EXITED
DEFAULT_HELP_LINK = "http://www.jython.org"

class Html_Browser(swing.JFrame):
################################################################################
# Function name: __init__
# Parameters:
#     -urlString: urlString to open in the browser
# Return:
#     An instance of the Html_Browser class.
# Description:
#     Creates a new instance of the Html_Browser.
################################################################################
    def __init__(self, urlString = DEFAULT_HELP_LINK):
        swing.JFrame.__init__(self, title="JESHelp Browser", size=(800, 600))
        self.contentPane.layout = java.awt.BorderLayout()
        self.contentPane.add(self.buildTopPane(urlString),
                java.awt.BorderLayout.NORTH)
        self.htmlPane = swing.JEditorPane(urlString, editable=0,
                hyperlinkUpdate=self.followHyperlink, size=(400,400))
        self.contentPane.add(swing.JScrollPane(self.htmlPane),
                java.awt.BorderLayout.CENTER)
        self.status = swing.JLabel(" ", preferredSize=(500,20))
        self.contentPane.add(self.status, java.awt.BorderLayout.SOUTH)

################################################################################
# Function name: buildTopPane
# Parameters:
#     -startUrl: Will build the top pane of the web browser
# Return: JPane
#
# Description:
#       Builds the top pane.
################################################################################
    def buildTopPane(self, startUrl):
        label = swing.JLabel("Go To:")
        self.field = swing.JTextField(preferredSize=(500,20),
                text=startUrl, actionPerformed=self.goToUrl)
        button = swing.JButton("Go", size=(100,100), 
                actionPerformed=self.goToUrl)
        topPane = swing.JPanel()
        topPane.add(label)
        topPane.add(self.field)
        topPane.add(button)
        return topPane

################################################################################
# Function name: goToUrl
# Parameters:
#     -event:
# Description: 
#        Sets the page displayed by the browser to the new page.
################################################################################
    def goToUrl(self, event):
        self.htmlPane.setPage(self.field.text)
        
################################################################################
# Function name: followHyperlink
# Parameters:
#     -hlEvent:
# Description: Sends the web browser to Hyper Links when the user clicks on them
#
################################################################################
    def followHyperlink(self, hlEvent):
        if hlEvent.eventType == ACTIVATED:
            self.htmlPane.setPage(hlEvent.URL)
            self.field.text = hlEvent.URL.toString()
        elif hlEvent.eventType == ENTERED:
            self.status.text = hlEvent.URL.toString()
        elif hlEvent.eventType == EXITED:
            self.status.text = " "
