#JES- Jython Environment for Students
#Copyright (C) 2002  Jason Ergle, Claire Bailey, David Raines, Joshua Sklare
#See JESCopyright.txt for full licensing information

import JESCommandWindow
import JESConstants
import java.awt as awt
import javax.swing as swing
import javax.swing.text.DefaultStyledDocument as DefaultStyledDocument
from pawt import colors

FALSE = (-1)

class JESCommandWindowDocument(DefaultStyledDocument):
################################################################################
# Function name: __init__
# Parameters:
#     -command: JESCommandWindow object that this object is associated with
# Return:
#     An instance of the JESCommandWindowDocument class.
# Description:
#     Creates an instance of the JESEditorDocument class.
################################################################################
    def __init__(self, command):
        self.command = command
        self.textAttrib = swing.text.SimpleAttributeSet()
        swing.text.StyleConstants.setFontSize(self.textAttrib,
                                              JESConstants.FONT_SIZE)
        swing.text.StyleConstants.setForeground(self.textAttrib, colors.white)

################################################################################
# Function name: insertString
# Parameters:
#     -offset: offset where the text will be inserted
#     -str: string that is being inserted
#     -a: attribute for the text that is being inserted.
# Description:
#     This function overrides the inherited insertString function.
################################################################################
    def insertString(self, offset, str, a):
        #When insertString is called by methods in JES, there is no need to
        #check the position
        if self.command.getIsSystem() != FALSE:
            DefaultStyledDocument.insertString(self,
                                               offset,
                                               str,
                                               self.textAttrib)
        else:
            #When insertString is called by Jython, check to make sure the
            #string is to be inserted after the most recent prompt
            if self.command.getCaretPosition() >= self.command.getOldPos():
                DefaultStyledDocument.insertString(self,
                                                   offset,
                                                   str,
                                                   self.textAttrib)

################################################################################
# Function name: remove
# Parameters:
#     -offset: offset of the text that is being removed
#     -len: length of the text that is being removed
# Description:
#     This function overrides the inherited remove function.  
################################################################################
    def remove(self, offset, len):
        #When remove is called by methods in JES, there is no need to check the
        #position
        if self.command.getIsSystem() != FALSE:
            DefaultStyledDocument.remove(self, offset, len)
        else:
            #When remove is called by Jython, check to make sure the string is
            #to be inserted after the most recent prompt
            if self.command.getCaretPosition() >= self.command.getOldPos():
                DefaultStyledDocument.remove(self, offset, len)

################################################################################
# Function name: getTextAttrib
# Return:
#     -textAttrib: the text attribute set.
# Description:
#     This method returns the document's text attrib variable.
################################################################################
    def getTextAttrib(self):
        return self.textAttrib
       
