#JES- Jython Environment for Students
#Copyright (C) 2002  Jason Ergle, Claire Bailey, David Raines, Joshua Sklare
#See JESCopyright.txt for full licensing information

#TODO - better type names
import java.awt as awt
import javax.swing as swing
import time
from java.awt.event import ActionListener

DEBUG_WINDOW_TITLE    = 'JES Debug Window #%s - %H:%M:%S'
DEBUG_WINDOW_SIZE     = (400,400)
CLOSE_BUTTON_CAPTION  = 'Close'
VAR_NAME_COL_CAPTION  = 'Variable'
VAR_TYPE_COL_CAPTION  = 'Type'
VAR_VALUE_COL_CAPTION = 'Value'

class JESDebugWindow(swing.JFrame, ActionListener):
################################################################################
# Function name: __init__
# Parameters:
#     -varsToDisplay: collection of variables to be displayed.
#     -windowNumber: window number (or ID) which can be used to differentiate
#                    one window from another.
# Return:
#     An instance of the JESDebugWindow class.
# Description:
#     Creates an instance of the JESDebugWindow class and displays it.
################################################################################
    def __init__(self, varsToDisplay, windowNumber):
        now = time.localtime(time.time())
        self.title = time.strftime(DEBUG_WINDOW_TITLE, now)
        self.title = self.title % windowNumber
        self.size = DEBUG_WINDOW_SIZE
        self.contentPane.setLayout(swing.BoxLayout(self.contentPane,
                                                   swing.BoxLayout.Y_AXIS))

        #Load the variables from varsToDisplay into an array for the JTable


        # TODO
        # replace varVal.__class__.__name__ with something more meaningful
        # and scan for things without "__class__" or "__name__" fields
        # some swing components don't have them

        varDict = self.__buildVarDict__(varsToDisplay)


        #Create the TableModel and JTable components
        dtm = swing.table.DefaultTableModel(varDict, [VAR_NAME_COL_CAPTION,
                                                      VAR_TYPE_COL_CAPTION,
                                                      VAR_VALUE_COL_CAPTION])
        varTable = swing.JTable(dtm)
        varTable.getColumnModel().getColumn(0).setPreferredWidth(1);

        #Create panels and button, and place them on the frame
        closeButton = swing.JButton(CLOSE_BUTTON_CAPTION, actionListener=self)
        topPane = swing.JScrollPane(varTable)
        bottomPanel = swing.JPanel()
        bottomPanel.add(closeButton)
        self.contentPane.add(topPane)
        self.contentPane.add(bottomPanel)

        self.setDefaultCloseOperation(1)
        self.setVisible(1)

################################################################################
# Function name: actionPerformed
# Parameters:
#     -event: event object that represents action that occured
# Description:
#     This function closes the debug window when the close button is pressed.  
################################################################################
    def actionPerformed(self, event):
        actionCommand = event.getActionCommand()
        if actionCommand == CLOSE_BUTTON_CAPTION:
            self.setVisible(0)

################################################################################
# Function name: __buildVarDict__
# Parameters:
#     -varsToDisplay: the dictionary of variable to display
# Return:
#     An array of three value collections (variable name, variable class name,
#     and variable value)
# Description:
#     Accepts a hash table of items and returns an an array.  The array is to
#     be shown in the debug window;  This will ignore variables that don't
#     have __class__ fields.
################################################################################
    def __buildVarDict__(self, varsToDisplay ):
        varDict = []
        for varName, varVal in varsToDisplay.items():
            
            try:
                varDict += [[varName, varVal.__class__.__name__, varVal]]

            except AttributeError:
                # some objects (swing objects, say) don't have the class field
                # this is a crude way to avoid that generating errors
                pass
            
        return varDict



