#JES- Jython Environment for Students
#Copyright (C) 2002  Jason Ergle, Claire Bailey, David Raines, Joshua Sklare
#See JESCopyright.txt for full licensing information

import JESAbout
import JESRunnable
import JESConstants
import JESDebugWindow
import JESInterpreter
import JESStdOutputBuffer
import JESUI
import java.io as io
import java.lang as lang
import javax.swing as swing
import string
import sys
import tabnanny
from code import compile_command

class JESProgram:
################################################################################
# Function name: __init__
# Return:
# Description:
#
################################################################################
    def __init__(self):
        self.textForCommandWindow = ''
        self.aboutWindow = None
        self.gui = JESUI.JESUI(self)
        self.filename = ''
        self.interpreter = JESInterpreter.JESInterpreter(self)
        self.chooser = swing.JFileChooser()
        self.setHelpArray()
        self.gui.show()

################################################################################
# Function name: newFile
# Description: 
#     Blanks outthe fileName and the text in the editor window. 
################################################################################
    def newFile(self):
        self.gui.editor.setText('')
        self.gui.setFileName('')
        self.chooser = swing.JFileChooser() #reset the filechooser to keep

################################################################################
# Function name: openFile
# Description: 
#     This function will open a dialog window allowing the user to select a 
#     file.  If the user selects a file, it is opened, and its contents are  
#     place into the text editor. 
################################################################################
    def openFile(self):
        self.chooser.setApproveButtonText("Open File")
        returnVal = self.chooser.showOpenDialog(self.gui)
        if returnVal ==  0: #User has chosen a file, so now it can be opened
            file = open(self.chooser.getSelectedFile().getPath(),'r')
            self.filename = file.name
            self.gui.setFileName(self.chooser.getSelectedFile().getName())
            self.gui.editor.setText(file.read())
            self.gui.editor.modified = 0
            file.close()
      
################################################################################
# Function name: saveFile
# Description:  
#     This function is called when the user selects the save file option from 
#     the user menu.  If there is no file opened yet, the function will do          
#     nothing.  If the file has no name, the function will call saveAs.  Else, 
#     it will write the contents of the text editor to the file of the current 
#     name.
################################################################################
    def saveFile(self):
        try:
            if self.filename != '' :
                self.checkTabs()
                text=self.gui.editor.getText()
                file = open(self.chooser.getSelectedFile().getPath(),'w')
                #David - testing something out
                file.write(text)
                self.filename = file.name
                file.close()
                self.gui.editor.modified = 0
            else:
                self.saveAs()
        except:
            #Error handling for saveFile
            self.saveAs()

################################################################################
# Function name: saveAs
# Description: 
#     Opens a dialog and allows users to select or either type in a filename to
#     save the file as.  If the user selects cancel, nothing happens.
################################################################################
    def saveAs(self):
       try:
        self.checkTabs()
        text = self.gui.editor.getText()
        self.chooser.setApproveButtonText("Save File")
        returnVal = self.chooser.showSaveDialog(self.gui)
        if returnVal ==  0: #User has chosen a file, so now it can be saved
            #DNR
            file = open(self.chooser.getSelectedFile().getPath(),'w')
            self.gui.setFileName(self.chooser.getSelectedFile().getName())
            file.write(text)
            self.filename = file.name
            file.close()
            self.gui.editor.modified = 0
       except:
         #Error handling for saveAs
         print "Error coming from saveAs"

################################################################################
# Function name: checkForSave
# Description:
#
################################################################################
    def checkForSave(self):
        pass

################################################################################
# Function name: loadFile
# Description:
#
################################################################################
    def loadFile(self):

        if self.filename == '':

            self.sendTextToCommandWindow( \
                JESConstants.JESPROGRAM_NO_FILE )
            
            self.interpreter.program.gui.commandWindow.restoreConsole('load')


        else:
            self.gui.commandWindow.setKeymap(None)
            self.checkTabs()
            try:
                file = open(self.filename, 'r')
                fileText = file.read()
                self.interpreter.load( fileText )

            except:
                self.sendTextToCommandWindow( \
                    JESConstants.JESPROGRAM_ERROR_LOADING_FILE + \
                    self.filename + '\n')
                self.interpreter.program.gui.commandWindow.restoreConsole('load')



################################################################################
# Function name: checkTabs
# Description:
#
################################################################################
    def checkTabs(self):
        if self.filename != '':
            #tabnanny sends the response to stdout, this redirects that output
            outputBuffer = JESStdOutputBuffer.JESStdOutputBuffer(sys.stdout)
            tabnanny.check(self.filename)
            outputBuffer.restoreOutput()
            errors = outputBuffer.getText()
            if errors == '':
                return
            else:
                self.sendTextToCommandWindow(errors)
 
################################################################################
# Function name: closeProgram
# Description:
#     Exits JES
################################################################################
    def closeProgram(self):
        lang.System.exit(0)

################################################################################
# Function name: stopThread
# Description:
#
################################################################################
    def stopThread(self):
        self.interpreter.stopThread()

################################################################################
# Function name: openAboutWindow
# Description: 
#     Opens up a JESAboutWindow.
################################################################################
    def openAboutWindow(self):
        if self.aboutWindow == None:
            aboutWindow = JESAbout.JESAbout()
        aboutWindow.show()
        
################################################################################
# Function Name: setHelpArray
# Description: 
#     This sets the array of help file names in self.gui.  If the folder 
#     JESHelp cannot be located, the function will throw an exception which is 
#     caught by the function.
################################################################################
    def setHelpArray(self):
        try:
            myFile= io.File("JESHelp")
            arrayOfFiles = myFile.listFiles()
            arrayOfNames = []
            for x in arrayOfFiles:
                arrayOfNames.append(x.toURL())
            stringNames = []
            for x in arrayOfNames:
                stringNames.append(x.toString())
            self.gui.SetHelpFiles(stringNames)
        except:
            print "ERROR opening help files"

################################################################################
# Function name: runCommand
# Parameters:
#     -text:
# Description:
#
################################################################################
    def runCommand(self, text):

        self.gui.commandWindow.setKeymap(None)
        try:
            text = text + '\n'

            code = compile_command(text)
        except SyntaxError:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            msg = self.interpreter.GetExceptionDescription( exc_type )


            self.sendErrorToCommandWindow( msg )
            self.gui.commandWindow.restoreConsole('run')
        else:
            self.interpreter.runCommand(code)

################################################################################
# Function name: sendTextToCommandWindow
# Parameters: 
#     -text:
# Description:
#     As the interpreter runs, it will generate text for the command window to 
#     display.  That text is buffered in JESProgram until the command window 
#     asks for it with the getTextForCommandWindow function
################################################################################
    def sendTextToCommandWindow(self, text):
        self.textForCommandWindow = self.textForCommandWindow + text

################################################################################
# Function name: getTextForCommandWindow
# Description: 
#     Returns the buffered text that was generated by JESInterpreter
################################################################################
    def getTextForCommandWindow(self):
        returnText = self.textForCommandWindow
        self.textForCommandWindow = ''
        return returnText

################################################################################
# Function name: sendErrorToCommandWindow
# Parameters:
#     -text:
# Description:
#
################################################################################
    def sendErrorToCommandWindow(self, text):
        self.textForCommandWindow = self.textForCommandWindow + text

################################################################################
# Function name: sendError
# Parameters:
#     -text:
# Description:
#
################################################################################
    def sendError(self):
        errFp = StringIO.StringIO()
        traceback.print_exc( file=errFp )
        errMsg = errFp.getvalue()
        self.sendErrorToCommandWindow( errMsg)

if __name__ == '__main__':
    mainJESProgram = JESProgram()


