#JES- Jython Environment for Students
#Copyright (C) 2002  Jason Ergle, Claire Bailey, David Raines, Joshua Sklare
#See JESCopyright.txt for full licensing information

import JESCommandWindow
import JESConstants
import JESDebugWindow
import JESEditor
import JESProgram
import Html_Browser
import java.awt as awt
import java.awt.Event as Event
import java.awt.event.KeyEvent as KeyEvent
import java.lang as lang
import os
from java.awt.event import ActionListener
from pawt import swing

MENU_SEPARATOR = '-'
COMMAND_NEW    = 'New'
COMMAND_OPEN   = 'Open'
COMMAND_SAVE   = 'Save'
COMMAND_SAVEAS = 'Save As...'
COMMAND_EXIT   = 'Exit'
COMMAND_CUT    = 'Cut'
COMMAND_COPY   = 'Copy'
COMMAND_PASTE  = 'Paste'
COMMAND_UNDO   = 'Undo'
COMMAND_HELP   = 'Help'
COMMAND_ABOUT  = 'About...'

FILE_TITLE = 'File'
EDIT_TITLE = 'Edit'
HELP_TITLE = 'Help'

HELP_FILE_EXTENTION = '.html'

#The following is an array that is used to build the main menu bar.  The
#information stored in here is the high level menu item names, the menu bar
#option names, and the accelerator keys for those menu options.
MENU_OPTIONS = [
    [FILE_TITLE,
      [[COMMAND_NEW,    KeyEvent.VK_N,  Event.CTRL_MASK],
       [COMMAND_OPEN,   KeyEvent.VK_O,  Event.CTRL_MASK],
       [COMMAND_SAVE,   KeyEvent.VK_S,  Event.CTRL_MASK],
       [COMMAND_SAVEAS, KeyEvent.VK_S,  Event.CTRL_MASK + Event.SHIFT_MASK],
       [MENU_SEPARATOR, 0,              0],
       [COMMAND_EXIT,   KeyEvent.VK_Q,  Event.CTRL_MASK]]],
    [EDIT_TITLE,
      [[COMMAND_CUT,    KeyEvent.VK_X,  Event.CTRL_MASK],
       [COMMAND_COPY,   KeyEvent.VK_C,  Event.CTRL_MASK],
       [COMMAND_PASTE,  KeyEvent.VK_V,  Event.CTRL_MASK],
       [COMMAND_UNDO,   KeyEvent.VK_Z,  Event.CTRL_MASK]]],
    [HELP_TITLE,
      [[COMMAND_ABOUT,  0,              0]]]]

LOAD_BUTTON_CAPTION = 'Load'
STOP_BUTTON_CAPTION = 'Stop'
UNTITLED_FILE_NAME  = 'Untitled'
HELP_URL = ''

MIN_COMMAND_WINDOW_SIZE = 150
MAX_COMMAND_WINDOW_SIZE = 250
VISUAL_CONTROL_MARGIN_SIZE = 5
SPLITTER_SIZE = 10
SPLITTER_LOCATION = 320
BUTTON_PANE_HEIGHT = 15
STATUS_BAR_HEIGHT = 25

PROMPT_SAVE_MESSAGE = 'You must save the file that you are working on before loading it.  Would you like to save now?'
PROMPT_SAVE_CAPTION = 'Save File?'

class JESUI(swing.JFrame, ActionListener):
################################################################################
# Function name: __init__
# Return:
#     An instance of the JESUI class.
# Description:
#     Creates a new instance of the JESUI.
################################################################################
    def __init__(self, program):
        self.swing = swing
        self.program  = program
        self.size = JESConstants.INITIAL_WINDOW_SIZE
        self.windowClosing = self.exit
        self.contentPane.setLayout(swing.BoxLayout(self.contentPane,
                                                   swing.BoxLayout.Y_AXIS))

        #Create the visual components that will be placed in the UI
        self.editor        = JESEditor.JESEditor(self)
        self.commandWindow = JESCommandWindow.JESCommandWindow(self)
        self.loadButton    = swing.JButton(LOAD_BUTTON_CAPTION,
                                           actionListener=self)
        self.stopButton    = swing.JButton(STOP_BUTTON_CAPTION,
                                           actionListener=self)
        self.cursorStatusLabel = swing.JLabel()
        self.cursorStatusLabel.setBorder(swing.BorderFactory.createEmptyBorder
                                                (0,
                                                 VISUAL_CONTROL_MARGIN_SIZE,
                                                 0,
                                                 VISUAL_CONTROL_MARGIN_SIZE))



        #Create and set up the panes that all visual components reside on
        splitterPane = swing.JSplitPane()
        editorPane   = swing.JScrollPane(self.editor)
        buttonPane   = swing.JPanel()
        commandPane  = swing.JScrollPane(self.commandWindow)
        bottomPane   = swing.JPanel()
        statusbar    = swing.JPanel()

        splitterPane.orientation = swing.JSplitPane.VERTICAL_SPLIT
        splitterPane.leftComponent = editorPane
        splitterPane.rightComponent = bottomPane
        splitterPane.setDividerSize(SPLITTER_SIZE)
        splitterPane.setDividerLocation(SPLITTER_LOCATION)

        editorPane.setPreferredSize(awt.Dimension(lang.Short.MAX_VALUE,
                                                  lang.Short.MAX_VALUE))

        buttonPane.setLayout(awt.BorderLayout())
        buttonPane.setBorder(swing.BorderFactory.createEmptyBorder
                                                 (VISUAL_CONTROL_MARGIN_SIZE,
                                                  VISUAL_CONTROL_MARGIN_SIZE,
                                                  VISUAL_CONTROL_MARGIN_SIZE,
                                                  VISUAL_CONTROL_MARGIN_SIZE))
        buttonPane.setMaximumSize(awt.Dimension(lang.Short.MAX_VALUE,
                                                BUTTON_PANE_HEIGHT))

        commandPane.setMinimumSize(awt.Dimension(0, MIN_COMMAND_WINDOW_SIZE))

        bottomPane.setLayout(swing.BoxLayout(bottomPane,
                                             swing.BoxLayout.Y_AXIS))

        statusbar.setMinimumSize(awt.Dimension(0, STATUS_BAR_HEIGHT))
        statusbar.setMaximumSize(awt.Dimension(lang.Short.MAX_VALUE,
                                               STATUS_BAR_HEIGHT))
        statusbar.setLayout(awt.BorderLayout())
        statusbar.setBorder(swing.BorderFactory.createLoweredBevelBorder())

        #Add all of the components to the main frame
        self.contentPane.add(splitterPane)
        self.contentPane.add(statusbar)

        bottomPane.add(buttonPane)
        bottomPane.add(commandPane)
        buttonPane.add(self.loadButton, awt.BorderLayout.WEST)
        buttonPane.add(self.stopButton, awt.BorderLayout.EAST)
        statusbar.add(self.cursorStatusLabel, awt.BorderLayout.WEST)

        #Create the menu bar and menu items
        self.menu = swing.JMenuBar()
        self.setJMenuBar(self.menu)

        for eachMenu in MENU_OPTIONS:
            newMenu = swing.JMenu(eachMenu[0], actionListener=self)
            self.menu.add(newMenu)
            #Create each menu option under the menu
            for eachMenuItem in eachMenu[1]:
                if eachMenuItem[0] == MENU_SEPARATOR:
                    newMenu.addSeparator()
                else:
                    newMenuItem = swing.JMenuItem(eachMenuItem[0],
                                                  actionListener=self)
                    if eachMenuItem[1] <> 0:
                        newMenuItem.setAccelerator(swing.KeyStroke.getKeyStroke
                                                           (eachMenuItem[1],
                                                            eachMenuItem[2],
                                                            0))
                    newMenu.add(newMenuItem)
            #If this is the help menu, store it in the self.helpMenu variable.
            if eachMenu[0] == HELP_TITLE:
                self.helpMenu = newMenu

        #Set remaining object variables
        self.heldText = ''
        self.setRunning(0)
        self.setFileName('')
        self.UpdateRowCol(1, 1)
        self.helpFiles = {}

################################################################################
# Function name: exit
# Parameters:
#     -event: event object that represents the event that occured
# Description:
#     This function, which closes the program, is called when the user closes
#     the JES program with the 'X' on the main window.
################################################################################
    def exit(self, event):
        self.program.closeProgram()

################################################################################
# Function name: actionPerformed
# Parameters:
#     -event: event object that represents action that occured
# Description:
#     This function is called when a menu option is selected or a button is
#     pressed.  It calls the correct function in order to perform the action
#     that the user wants.
################################################################################
    def actionPerformed(self, event):
        actionCommand = event.getActionCommand()

        if actionCommand == COMMAND_NEW:
            self.program.newFile()
        elif actionCommand == COMMAND_OPEN:
            self.program.openFile()
        elif actionCommand == COMMAND_SAVE:
            self.program.saveFile()
        elif actionCommand == COMMAND_SAVEAS:
            self.program.saveAs()
        elif actionCommand == COMMAND_EXIT:
            self.program.closeProgram()
        elif actionCommand == COMMAND_CUT:
            self.cut()
        elif actionCommand == COMMAND_COPY:
            self.copy()
        elif actionCommand == COMMAND_PASTE:
            self.paste()
        elif actionCommand == COMMAND_UNDO:
            self.undo()
        elif actionCommand == COMMAND_HELP:
            self.openBrowser(self, HELP_URL)
        elif actionCommand == COMMAND_ABOUT:
            self.program.openAboutWindow()
        elif actionCommand == LOAD_BUTTON_CAPTION:
            if self.editor.modified:
                if self.promptSave():
                    self.program.loadFile()
            else:
                self.program.loadFile()
        elif actionCommand == STOP_BUTTON_CAPTION:
            self.program.stopThread()
        else:
            self.CheckIfHelpTopic(actionCommand)

################################################################################
# Function name: promptSave
# Return:
#     TRUE if the file was saved successfully, FALSE if the save failed or the
#     user decided not to save the file.
# Description:
#     Prompts the user whether they want to save the currently loaded file.
################################################################################
    def promptSave(self):
        promptResult = swing.JOptionPane.showConfirmDialog(
                                           self,
                                           PROMPT_SAVE_MESSAGE,
                                           PROMPT_SAVE_CAPTION,
                                           swing.JOptionPane.CANCEL_OPTION
                                            + swing.JOptionPane.YES_OPTION)

        if promptResult == swing.JOptionPane.YES_OPTION:
            return self.program.saveFile()
        else:
            return 0

################################################################################
# Function name: openBrowser
# Parameters:
#     -url: Target URL or file name for the browser.  If a file name is given,
#           the entire path to that file must also be included.
# Description:
#     Opens a browser window to the specified location.  This is used when
#     displaying the HTML help.
################################################################################
    def openBrowser(self, target):
        try:
           j=Html_Browser.Html_Browser(target)
           j.show()
        except:
           print "ERROR opening broswer with file:",target

################################################################################
# Function name: setRunning
# Parameters:
#     -runBool: Boolean identifying whether the program is running.
# Description:
#     This function is called to tell the GUI whether the Jython interpreter is
#     running any code.  When running, the GUI will enable the stop button,
#     disable the load button, and change the cursor to an hourglass.
################################################################################
    def setRunning(self, runBool):
        self.running = runBool
        self.loadButton.enabled = not runBool
        self.stopButton.enabled = runBool

        if runBool:
            cursor = awt.Cursor(awt.Cursor.WAIT_CURSOR)
            textCursor = cursor
        else:
            cursor = awt.Cursor(awt.Cursor.DEFAULT_CURSOR)
            textCursor = awt.Cursor(awt.Cursor.TEXT_CURSOR)

        self.setCursor(cursor)
        self.editor.setCursor(textCursor)
        self.commandWindow.setCursor(textCursor)

################################################################################
# Function name: setFileName
# Parameters:
#     -filename: name of the currenly open file 
# Description:
#     Sets the file name that is displayed in the program caption.  If the
#     filename parameter is set to '', then the file name will be shown as
#     'Untitled'.
################################################################################
    def setFileName(self, filename):
        if filename == '':
            filename = UNTITLED_FILE_NAME

        self.title = JESConstants.APPLICATION_TITLE % filename

################################################################################
# Function name: callTextEditFunction
# Description:
#     Performs the cut operation on the either the editor or command window,
#     depending on which one has the focus.
################################################################################
    def callTextEditFunction(self, function):
        focusedComponent = self.getFocusOwner()

        if isinstance(focusedComponent, swing.JTextPane):
            focusedComponent.function()

################################################################################
# Function name: cut
# Description:
#     Performs the cut operation on the either the editor or command window,
#     depending on which one has the focus.
################################################################################
    def cut(self):
        focusedComponent = self.getFocusOwner()

        if isinstance(focusedComponent, swing.JTextPane):
            focusedComponent.cut()

################################################################################
# Function name: copy
# Description:
#     Performs the copy operation on the either the editor or command window,
#     depending on which one has the focus.
################################################################################
    def copy(self):
        focusedComponent = self.getFocusOwner()

        if isinstance(focusedComponent, swing.JTextPane):
            focusedComponent.copy()

################################################################################
# Function name: paste
# Description:
#     Performs the paste operation on the either the editor or command window,
#     depending on which one has the focus.
################################################################################
    def paste(self):
        focusedComponent = self.getFocusOwner()

        if isinstance(focusedComponent, swing.JTextPane):
            focusedComponent.paste()

################################################################################
# Function name: undo
# Description:
#     Performs the undo operation on the either the editor or command window,
#     depending on which one has the focus.
################################################################################
    def undo(self):
        focusedComponent = self.getFocusOwner()

        if isinstance(focusedComponent, swing.JTextPane):
            focusedComponent.undo()

################################################################################
# Function name: UpdateRowCol
# Parameters:
#     -row: current row that the cursor is on
#     -col: current column that the cursor is on
# Description:
#     Updates the status bar with the given row and column.
################################################################################
    def UpdateRowCol(self, row, col):
        self.cursorStatusLabel.text = str(col) + ': ' + str(row)

################################################################################
# Function name: SetHelpFiles
# Parameters:
#     -helpFiles: array of file names (including entire path) for all help files
#                 that will be added to help menu.
# Description:
#     Adds the help files specified in the helpFiles parameter to the help menu.
################################################################################
    def SetHelpFiles(self, helpFiles):
        self.helpFiles = {}
        self.helpMenu.addSeparator()
        for eachHelpFilePath in helpFiles:
            fileName = os.path.basename(eachHelpFilePath)
            fileName = fileName.replace(HELP_FILE_EXTENTION, '')
            self.helpFiles[fileName] = eachHelpFilePath
            newMenuItem = swing.JMenuItem(fileName,
                                          actionListener=self)
            self.helpMenu.add(newMenuItem)

################################################################################
# Function name: CheckIfHelpTopic
# Parameters:
#     -helpTopic: string containing the name of the menu option to be checked
# Return:
#     Returns true if the helpTopic variable was the name of a help topic, or
#     false if it was not.
# Description:
#     Checks to see if the given string (specified in the helpTopic parameter)
#     is the name of a help topic.  If it is, the openBrowser function is called
#     to open up that help file.
################################################################################
    def CheckIfHelpTopic(self, helpTopic):
        if self.helpFiles.has_key(helpTopic):
            self.openBrowser(self.helpFiles[helpTopic])
            return 1
        else:
            return 0
